<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:java="http://xml.apache.org/xslt/java"
	xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
	xmlns:gslib="http://www.greenstone.org/skinning"
	xmlns:gsf="http://www.greenstone.org/greenstone3/schema/ConfigFormat"
	extension-element-prefixes="java util"
	exclude-result-prefixes="java util gsf">


  <xsl:variable name="httpDocument">
    <xsl:value-of select="$library_name"/>
    <xsl:text>/collection/</xsl:text>
    <xsl:value-of select="/page/pageResponse/collection/@name"/>
    <xsl:text>/document/</xsl:text>
    <xsl:value-of select="/page/pageRequest/paramList/param[@name = 'd']/@value"/> <!-- or use $docID from default/transform/pages/document.xsl ???? -->
  </xsl:variable>


  <xsl:template name="iiif-links">
    <xsl:param name="identifier"/>

    <gsf:variable name="iiifImageRootIdentifier">
      <xsl:text>/gs-cantaloupe/iiif/2/</xsl:text><xsl:value-of select="$site_name"/>:<xsl:value-of select="$collNameChecked"/>:<xsl:value-of select="$identifier"/>
    </gsf:variable>

    <!--
    <gsf:variable name="iiifPresentationRootIdentifier"><xsl:value-of select="$docID"/></gsf:variable>
    -->
    
    <gsf:variable name="iiifPresentationRootIdentifier">http-greenstone://<xsl:value-of select="$site_name"/>/<xsl:value-of select="$collNameChecked"/>/<xsl:value-of select="$docID"/></gsf:variable>

    <!--
    <div style="padding-top: 10px; padding-bottom: 10px;">
      IIIF Server Info URL:
      <a href="/gs-cantaloupe/iiif/2/{$site_name}:{$collNameChecked}:{$identifier}/info.json">
	/gs-cantaloupe/iiif/2/<xsl:value-of select="$site_name"/>:<xsl:value-of select="$collNameChecked"/>:<xsl:value-of select="$identifier"/>/info.json
	</a> 
    </div>

    <div style="padding-top: 10px; padding-bottom: 10px;">
      IIIF Server Image URL:
      <a href="/gs-cantaloupe/iiif/2/{$site_name}:{$collNameChecked}:{$identifier}/full/full/0/default.jpg">
	/gs-cantaloupe/iiif/2/<xsl:value-of select="$site_name"/>:<xsl:value-of select="$collNameChecked"/>:<xsl:value-of select="$identifier"/>/full/full/0/default.jpg
      </a>
    </div>

    <div style="padding-top: 10px; padding-bottom: 10px;">
      Single Page Manifest
      <a href="{$httpDocument}?book=mirador&amp;excerptid=run-iiif-manifest">
	<xsl:value-of select="$httpDocument"/>?excerptid=run-iiif-manifest
      </a>
    </div>
    -->
    
    <gsf:script>
      var chosen_title = "untitled";

      if (gs.variables.chosenTitle &amp;&amp; !gs.variables.docNodeID) {
          // simple doc
          chosen_title = gs.variables.chosenTitle;
      }
      else {
          var docNodeID = gs.variables.docNodeID;
          var page_pos = docNodeID.indexOf('.');      
          chosen_title = (page_pos>=0) ? "page " + docNodeID.substring(page_pos+1) : docNodeID;
      }
            
      var iiif_doc_image = {
	"id": document.location.origin + gs.variables.iiifImageRootIdentifier,
	//"format": mime_type, // ???? or is it OK to always have this as image/jpeg and rely on IIIF Image Server coverting as/when needed???
	//"chosen_title": gs.variables.chosenTitle,
	"chosen_title": chosen_title,
        "x_dim": gs.variables.imageWidth,
        "y_dim": gs.variables.imageHeight,
      };
      
      IIIF_Doc_Images.push(iiif_doc_image);
      
    </gsf:script>

  </xsl:template>


  <xsl:template name="googlevision-ocr-json">
    <xsl:param name="assocfilepath"/>
    
    <gsf:variable name="GVDocumentOCRJSON"><gsf:metadata name="GVDocumentOCRJSON"/></gsf:variable>
    <gsf:variable name="docSectionID"><xsl:value-of select="util:replace(@nodeID, '.', '_')"/></gsf:variable>

    <!--
    <div style="padding-top: 10px; padding-bottom: 10px;">
      Google Vision OCR JSON:
      <a>
	<xsl:attribute name="href">
	  <xsl:value-of disable-output-escaping="yes" select="/page/pageResponse/collection/metadataList/metadata[@name = 'httpPath']"/>
	  <xsl:text>/index/assoc/</xsl:text><xsl:value-of disable-output-escaping="yes" select="$assocfilepath" /><xsl:text>/</xsl:text><xsl:value-of select="$GVDocumentOCRJSON"/>
	</xsl:attribute>
	<xsl:value-of select="$GVDocumentOCRJSON"/>
      </a>
    </div>
    -->
    <gsf:script>
      load_gv_dococr_json(gs.variables.GVDocumentOCRJSON,gs.variables.docSectionID,display_gv_ocr_bounding_boxes);
    </gsf:script>
  </xsl:template>


  <xsl:template name="wrappedSectionPost">

    <!-- add in IIIF Server link for section image -->
    <xsl:variable name="nodeID" select="@nodeID"/>
    <xsl:call-template name="iiif-links">
      <xsl:with-param name="identifier" select="$nodeID" />
    </xsl:call-template>

    <!-- add in Google Vision OCR JSON link for section image -->
    <xsl:call-template name="googlevision-ocr-json">
      <xsl:with-param name="assocfilepath"><gsf:metadata name="assocfilepath" select="root"/></xsl:with-param>
    </xsl:call-template>    

  </xsl:template>

  <xsl:template name="sectionImage">
    <xsl:if test="$bookswitch != 'mirador'">
      <gsf:image type="screen"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="sectionContent">

    <xsl:choose>
      <xsl:when test="$bookswitch = 'mirador'">
	<gsf:variable name="screenImageWidth"><gsf:metadata name="ScreenWidth"/></gsf:variable>
	<gsf:variable name="screenImageHeight"><gsf:metadata name="ScreenHeight"/></gsf:variable>
	<gsf:variable name="imageWidth"><gsf:metadata name="ImageWidth"/></gsf:variable>
	<gsf:variable name="imageHeight"><gsf:metadata name="ImageHeight"/></gsf:variable>
	<gsf:variable name="docNodeID"><xsl:value-of select="@nodeID"/></gsf:variable> <!-- used to make each 'tidyDiv' unique -->
      </xsl:when>

      <xsl:otherwise>
	<xsl:call-template name="wrappedSectionImage"/>
	
	<div style="padding: 10px 0px 10px 0px">Ocr'd Text:</div>
	<i>
	  <xsl:call-template name="wrappedSectionText"/>
	</i>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:call-template name="wrappedSectionPost"/>
  </xsl:template>
  
  <xsl:template name="sectionContentForEditing">
    <xsl:call-template name="wrappedSectionImage"/>
    <xsl:call-template name="wrappedSectionTextForEditing"/>
  </xsl:template>

  <xsl:template name="documentPre">
    <xsl:variable name="httpCollection">library/sites/<xsl:value-of select="$site_name"/>/collect/<xsl:value-of select="/page/pageResponse/collection/@name"/></xsl:variable>
    <gsf:variable name="chosenTitle"><xsl:call-template name="choose-title"/></gsf:variable>
    <!-- <gsf:variable name="toplevelTitle"><gsf:metadata name="Title" select="root"/></gsf:variable> -->
    <gsf:variable name="imageWidth"><gsf:metadata name="ImageWidth"/></gsf:variable>
    <gsf:variable name="imageHeight"><gsf:metadata name="ImageHeight"/></gsf:variable>

    <gsf:script>
      console.log("**** chosenTitle = " + gs.variables.chosenTitle);
    </gsf:script>
    <gsf:script src="{$httpCollection}/js/document_viewer.js"/>
    <style>
       div.ocr-boundingbox-overlay {
         position: absolute;
         left: 0;
         top: 0;
       }

       div.ocr-boundingbox {
         position: absolute;
         border: solid black 1px;
       }

    </style>

    <xsl:if test="$bookswitch = 'mirador'">
      <!-- <gsf:script src="https://unpkg.com/mirador@latest/dist/mirador.min.js"/> -->
      
      <gsf:script src="ext/structured-image/mirador3/dist/mirador-with-annotations.js"/>

      
      <!-- Mirador, by default, uses Roboto font, so load it in -->
      <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto:300,400,500" />
    </xsl:if>
    
    <gsf:script>
      var IIIF_Doc_Images = [];
    </gsf:script>
     
  </xsl:template>
      

  <xsl:template name="documentContentMirador3">
    <xsl:if test="@docType='simple'">
      <gsf:script>$('#tocLoadingImage').hide()</gsf:script>
      <!-- add in IIIF Server link for simple page image -->
      <xsl:call-template name="iiif-links">
	<xsl:with-param name="identifier" select="$docID" />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="documentContent">
    <!-- The way document.xsl tempaltes are structured, only get to here is if @docType = 'simple' -->
    <!-- A 'paged' (or 'hierarical') document goes throgh 'wrappedContent' -->
    
    <div id="gs-document">

      <xsl:choose>
	<xsl:when test="$bookswitch = 'mirador'">
	  <xsl:call-template name="documentPre"/>
	  <xsl:call-template name="documentContentMirador3"/>
	  <xsl:call-template name="documentPost"/>
	  <xsl:call-template name="documentPostMirador3"/>
	</xsl:when>

	<xsl:otherwise>
	  <xsl:call-template name="documentPre"/>
	  <xsl:call-template name="wrappedSectionImage"/>
	  <div id="gs-document-text">
	    <xsl:call-template name="documentNodeText"/>
	  </div>
	  <xsl:call-template name="documentPost"/>
	</xsl:otherwise>

      </xsl:choose>
    </div>
      
  </xsl:template>

  
  <xsl:template name="documentPost">
    
    <xsl:if test="@docType='simple'">

      <!--
      <gsf:switch>
	<gsf:metadata name='HasGoogleVisionDocumentOCRJSON";'/>
      <gsf:when test='equals' test-value="1">
      -->

	  <!-- add in Google Vision OCR JSON link for simple page image -->      
	  <xsl:call-template name="googlevision-ocr-json">
	    <xsl:with-param name="assocfilepath"><gsf:metadata name="assocfilepath" pos="first"/></xsl:with-param>
	  </xsl:call-template>
	  <!--
	</gsf:when>
      </gsf:switch>
-->
      
    </xsl:if>
  </xsl:template>

  <xsl:template name="documentPostMirador3">

    <!-- Container element of Mirador whose id should be passed to the instantiating call as "id" -->    
    <div style="position: relative; width: 100%; height: 800px;">
      <div id="mirador3-viewer"> <xsl:comment>filler</xsl:comment></div>
    </div>

    <gsf:script>      
      var iiifpres_root_id = gs.variables.iiifPresentationRootIdentifier;
      console.log(gs);
      var iiifpres_label   = (gs.variables.toplevelTitle) ? gs.variables.toplevelTitle : gs.variables.chosenTitle;

      var iiif_manifest = initIIIFManifest(iiifpres_root_id,iiifpres_label, IIIF_Doc_Images);

      var mirador3 = createMirador3Viewer();
    </gsf:script>
    
  </xsl:template>

  <xsl:template name="wrappedDocumentTopLevel">
    <gsf:variable name="toplevelTitle"><xsl:value-of disable-output-escaping="yes" select="/page/pageResponse/document/documentNode/metadataList/metadata[@name='Title']" /></gsf:variable>
    <xsl:choose>
      <xsl:when test="$bookswitch = 'mirador'">
	<!-- <xsl:call-template name="documentPre"/> -->
	<div style="display:none;">
	  <xsl:call-template name="wrappedDocument"/> <!-- without ed=1, this template misses the binds in of documentPre and documentPost -->
	</div>
	<!-- <xsl:call-template name="documentPost"/> -->
	  <xsl:call-template name="documentPostMirador3"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:call-template name="wrappedDocument"/> <!-- without ed=1, this template misses the binds in of documentPre and documentPost -->
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  
	<!-- content of a simple document. Will not be used for editing mode -->
	<xsl:template name="documentPostInProgress">


			<!--
			canDoEditing = <xsl:value-of select="$canDoEditing"/> <br />
			editingTurnedOn = <xsl:value-of select="$editingTurnedOn"/>
			-->
			
			  <!--
			
			<xsl:choose>
			  <xsl:when test="$canDoEditing = 'true' and $editingTurnedOn = 'true'">
			  
			  <xsl:when test="$canDoEditing = 'true'">
-->			      
			    <gsf:script src="https://unpkg.com/mirador@latest/dist/mirador.min.js"/>


			    <!-- By default uses Roboto font. Be sure to load this or change the font -->
			    <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto:300,400,500" />
			    <!-- Container element of Mirador whose id should be passed to the instantiating call as "id" -->
			    
			    <div style="position: relative; width: 100%; height: 800px;">
			      <div id="my-mirador"> <xsl:comment>filler</xsl:comment></div>
			    </div>
			    
			    <gsf:script>
			      
var mirador = Mirador.viewer({
  "id": "my-mirador",
  "manifests": {
    "https://intermuse.sowemustthink.space/greenstone3/sample-manifest.json": {
      "provider": "IntermusE"
    }
    },
  "window" : { "panels" : { "annotations": true, "attribution": false } },
  "annotations": {
    "htmlSanitizationRuleSet": 'iiif', // See src/lib/htmlRules.js for acceptable values
    "filteredMotivations": ['oa:commenting', 'oa:tagging', 'sc:painting', 'commenting', 'tagging'],
  },    
  "windows": [
    {
      "loadedManifest": "https://intermuse.sowemustthink.space/greenstone3/sample-manifest.json",
      "canvasIndex": 1,
      "thumbnailNavigationPosition": 'far-bottom'
    }
  ]
});
			    </gsf:script>

			    <!--
			  </xsl:when>
			</xsl:choose>
-->
	  
	</xsl:template>
	
	
</xsl:stylesheet>
